from xml.dom.minidom import Document

class GraphXML:
    def __init__(self, order, size):
        self.doc = Document()
  
        graph = self.doc.createElement("Graph")
        graph.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance")
        graph.setAttribute("xsi:noNamespaceSchemaLocation", "Graph.xsd")
        self.doc.appendChild(graph)

        norder = self.doc.createElement("Order")
        nsize = self.doc.createElement("Size")
        self.nodes = self.doc.createElement("Nodes")
        self.edges = self.doc.createElement("Edges")
        
        norder.appendChild(self.doc.createTextNode(str(order)))
        nsize.appendChild(self.doc.createTextNode(str(size)))

        graph.appendChild(norder)
        graph.appendChild(nsize)
        graph.appendChild(self.nodes)
        graph.appendChild(self.edges)    

    def addNode(self, nodeId, label):
        element = self.doc.createElement("Node")
        element.setAttribute("Id", str(nodeId))        
        element.appendChild(self.doc.createTextNode(str(label)))
        self.nodes.appendChild(element)

    def addEdge(self, edgeId, fromNode, toNode):
        element = self.doc.createElement("Edge")
        element.setAttribute("Id", str(edgeId))
        element.setAttribute("From", str(fromNode))
        element.setAttribute("To", str(toNode))                        
        self.edges.appendChild(element)
      
    def xml(self):
        # prettyprint adds whitespace in textnodes (whitespace is significant in XML)
        # http://www.oracle.com/technology/pub/articles/wang-whitespace.html
        return self.doc.toxml("utf-8")

if __name__ == '__main__':
    graph = GraphXML(3, 3)
    graph.addNode(0, "1st node")  
    graph.addNode(1, "2nd node")
    graph.addNode(2, "3rd node")
    graph.addEdge(0, 0, 1)
    graph.addEdge(1, 0, 2)
    graph.addEdge(2, 1, 2)            
    print graph.xml()
