from xml.dom.minidom import Document

class FixedLayoutXML:
    def __init__(self, x, y, z):
        self.doc = Document()
  
        layout = self.doc.createElement("Layout")
        layout.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance")
        layout.setAttribute("xsi:noNamespaceSchemaLocation", "Layout.xsd")
        self.doc.appendChild(layout)
        
        fix = self.doc.createElement("FixedLayout")
        layout.appendChild(fix)
        
        origin = self.doc.createElement("Origin")
        origin.setAttribute("x", str(x))
        origin.setAttribute("y", str(y))
        origin.setAttribute("z", str(z))
        fix.appendChild(origin)
         
        self.nodes = self.doc.createElement("Nodes")
        fix.appendChild(self.nodes)

    def addNode(self, nodeId, x, y, z):
        element = self.doc.createElement("Node")
        element.setAttribute("Id", str(nodeId))        
        element.setAttribute("x", str(x))
        element.setAttribute("y", str(y))
        element.setAttribute("z", str(z))
        self.nodes.appendChild(element)
                
    def xml(self):
        # prettyprint adds whitespace in textnodes (whitespace is significant in XML)
        # http://www.oracle.com/technology/pub/articles/wang-whitespace.html
        return self.doc.toxml("utf-8")


class SpringLayoutXML:
    def __init__(self):
        self.doc = Document()
  
        layout = self.doc.createElement("Layout")
        layout.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance")
        layout.setAttribute("xsi:noNamespaceSchemaLocation", "Layout.xsd")
        self.doc.appendChild(layout)
        
        sl = self.doc.createElement("SpringLayout")
        layout.appendChild(sl)

        self.adaptiveDamping = self.doc.createElement("AdaptiveDamping")
        self.damping = self.doc.createElement("Damping")
        self.timeStep = self.doc.createElement("TimeStep")
        self.maxNodeVelocityThreshold = self.doc.createElement("MaxNodeVelocityThreshold")
        self.nodeRepulsion = self.doc.createElement("NodeRepulsion")
        self.maxRepulsionDistance = self.doc.createElement("MaxRepulsionDistance")

        sl.appendChild(self.adaptiveDamping)
        sl.appendChild(self.damping)
        sl.appendChild(self.timeStep)
        sl.appendChild(self.maxNodeVelocityThreshold)
        sl.appendChild(self.nodeRepulsion)
        sl.appendChild(self.maxRepulsionDistance)
        
        # default values
        self.adaptiveDamping.appendChild(self.doc.createTextNode("1"))
        self.damping.appendChild(self.doc.createTextNode("0.9"))
        self.timeStep.appendChild(self.doc.createTextNode("0.1"))
        self.maxNodeVelocityThreshold.appendChild(self.doc.createTextNode("0.08"))
        self.nodeRepulsion.appendChild(self.doc.createTextNode("0.9"))
        self.maxRepulsionDistance.appendChild(self.doc.createTextNode("1.9"))
        
        self.nodes = self.doc.createElement("Nodes")
        sl.appendChild(self.nodes)

        self.edges = self.doc.createElement("Edges")
        sl.appendChild(self.edges)

    def setAdaptiveDamping(self, val):
        self.adaptiveDamping.replaceChild(self.doc.createTextNode(str(val)), self.showNodes.firstChild)

    def setDamping(self, val):
        self.damping.replaceChild(self.doc.createTextNode(str(val)), self.showEdges.firstChild)

    def setTimeStep(self, val):
        self.timeStep.replaceChild(self.doc.createTextNode(str(val)), self.showNodes.firstChild)

    def setMaxNodeVelocityThreshold(self, val):
        self.maxNodeVelocityThreshold.replaceChild(self.doc.createTextNode(str(val)), self.showEdges.firstChild)

    def setNodeRepulsion(self, val):
        self.nodeRepulsion.replaceChild(self.doc.createTextNode(str(val)), self.showNodes.firstChild)

    def setMaxRepulsionDistance(self, val):
        self.maxRepulsionDistance.replaceChild(self.doc.createTextNode(str(val)), self.showEdges.firstChild)

    def addNode(self, nodeId, zPlane):
        # all nodes have the same repulsve force
        element = self.doc.createElement("Node")
        element.setAttribute("Id", str(nodeId))        
        element.setAttribute("zPlane", str(zPlane))
        self.nodes.appendChild(element)
                
    def addEdge(self, edgeId, length, stiffness):
        element = self.doc.createElement("Edge")
        element.setAttribute("Id", str(edgeId))        
        element.setAttribute("SpringLength", str(length))
        element.setAttribute("SpringStiffness", str(stiffness))        
        self.edges.appendChild(element)

    def xml(self):
        # prettyprint adds whitespace in textnodes (whitespace is significant in XML)
        # http://www.oracle.com/technology/pub/articles/wang-whitespace.html
        return self.doc.toxml("utf-8")


if __name__ == '__main__':
    # layout = FixedLayoutXML(0, 0, 20)
    # layout.addNode(0, 1.4, 5.4, 34)
    # layout.addNode(1, 2.4, 1.4, 4)
    # layout.addNode(2, 45.4, 3.4, 45)        
    # print layout.xml()
      
    layout = SpringLayoutXML()
    layout.addNode(0, 1)
    layout.addNode(1, 1)
    layout.addNode(2, 4)        
    layout.addEdge(0, 0.9, 5.0)
    print layout.xml()