from xml.dom.minidom import Document

class ShapesXML:
    def __init__(self):
        self.doc = Document()
  
        shapes = self.doc.createElement("Shapes")
        shapes.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance")
        shapes.setAttribute("xsi:noNamespaceSchemaLocation", "Shapes.xsd")
        self.doc.appendChild(shapes)

        self.showNodes = self.doc.createElement("ShowNodes")
        self.showEdges = self.doc.createElement("ShowEdges")
        self.definitions = self.doc.createElement("Definitions")
        self.nodes = self.doc.createElement("Nodes")

        self.labelsCentered = self.doc.createElement("LabelsCentered")
        self.wiredWidth = self.doc.createElement("WiredWidth")
        self.edgesWidth = self.doc.createElement("EdgesWidth")
        self.labelsWidth = self.doc.createElement("LabelsWidth")
        self.labelsOffsetX = self.doc.createElement("LabelsOffsetX")
        self.labelsOffsetY = self.doc.createElement("LabelsOffsetY")
        self.labelsSize = self.doc.createElement("LabelsSize")

        # default
        self.showNodes.appendChild(self.doc.createTextNode("1"))
        self.showEdges.appendChild(self.doc.createTextNode("1"))
        self.labelsCentered.appendChild(self.doc.createTextNode("1"))
        self.wiredWidth.appendChild(self.doc.createTextNode("1.4"))
        self.edgesWidth.appendChild(self.doc.createTextNode("1.4"))
        self.labelsWidth.appendChild(self.doc.createTextNode("2"))
        self.labelsOffsetX.appendChild(self.doc.createTextNode("0"))
        self.labelsOffsetY.appendChild(self.doc.createTextNode("0"))
        self.labelsSize.appendChild(self.doc.createTextNode("1"))

        shapes.appendChild(self.showNodes)
        shapes.appendChild(self.showEdges)
        shapes.appendChild(self.labelsCentered)
        shapes.appendChild(self.wiredWidth)
        shapes.appendChild(self.edgesWidth)
        shapes.appendChild(self.labelsWidth)
        shapes.appendChild(self.labelsOffsetX)
        shapes.appendChild(self.labelsOffsetY)
        shapes.appendChild(self.labelsSize)
        shapes.appendChild(self.definitions)
        shapes.appendChild(self.nodes)   
          

    def setShowNodes(self, val):
        self.showNodes.replaceChild(self.doc.createTextNode(str(val)), self.showNodes.firstChild)

    def setShowEdges(self, val):
        self.showEdges.replaceChild(self.doc.createTextNode(str(val)), self.showEdges.firstChild)

    def setLabelsCentered(self, val):
        self.labelsCentered.replaceChild(self.doc.createTextNode(str(val)), self.labelsCentered.firstChild)

    def setWiredWidth(self, val):
        self.wiredWidth.replaceChild(self.doc.createTextNode(str(val)), self.wiredWidth.firstChild)

    def setEdgesWidth(self, val):
        self.edgesWidth.replaceChild(self.doc.createTextNode(str(val)), self.edgesWidth.firstChild)

    def setLabelsWidth(self, val):
        self.labelsWidth.replaceChild(self.doc.createTextNode(str(val)), self.labelsWidth.firstChild)

    def setLabelsOffsetX(self, val):
        self.labelsOffsetX.replaceChild(self.doc.createTextNode(str(val)), self.labelsOffsetX.firstChild)

    def setLabelsOffsetY(self, val):
        self.labelsOffsetY.replaceChild(self.doc.createTextNode(str(val)), self.labelsOffsetY.firstChild)

    def setLabelsSize(self, val):
        self.labelsSize.replaceChild(self.doc.createTextNode(str(val)), self.labelsSize.firstChild)

    def addShape(self, sid, shape, red, green, blue, size, stretchX, stretchY, stretchZ):
        # Shapes and colors are hard coded for the time being.
        #     ShapeId values can be one of the following:
        #         0 - Solid Sphere (size is radius)
        #         1 - Solid Cube (size is the length of a side)
        #         2 - Solid Torus (size is external radous)
        #         3 - Wireframe Sphere (non-selectable)
        #         4 - Wirefrane Cube (non-selectable)
        #         5 - Wireframe Torus (non-selectable)
        element = self.doc.createElement("Shape")
        element.setAttribute("Id", str(sid))
        element.setAttribute("ShapeId", str(shape))        
        element.setAttribute("Red", str(red))
        element.setAttribute("Green", str(green))
        element.setAttribute("Blue", str(blue))
        element.setAttribute("Size", str(size))
        element.setAttribute("StretchX", str(stretchX))
        element.setAttribute("StretchY", str(stretchY))
        element.setAttribute("StretchZ", str(stretchZ))        
        self.definitions.appendChild(element)

    def addNode(self, nodeId, shapeId):
        element = self.doc.createElement("Node")
        element.setAttribute("Id", str(nodeId))        
        element.setAttribute("Shape", str(shapeId))   
        self.nodes.appendChild(element)
      
    def xml(self):
        # prettyprint adds whitespace in textnodes (whitespace is significant in XML)
        # http://www.oracle.com/technology/pub/articles/wang-whitespace.html
        return self.doc.toxml("utf-8")

if __name__ == '__main__':
    shapes = ShapesXML()
    shapes.setShowNodes(1)
    shapes.setShowEdges(1)
    shapes.addShape(0, 0, 0.690, 1.000, 0.310, 0.2, 1, 1, 1)
    shapes.addShape(1, 0, 0.396, 0.773, 1.000, 0.2, 1, 1, 1)  
    shapes.addNode(0, 0)  
    shapes.addNode(1, 1)
    shapes.addNode(2, 1)    
    print shapes.xml()
    
